package tk.mybatis.incomplete;

import org.apache.ibatis.io.Resources;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.junit.Before;

import java.io.IOException;
import java.io.Reader;
import java.net.URL;
import java.sql.Connection;

/**
 * @author liuzh
 */
public abstract class BaseTest {
    private SqlSessionFactory sqlSessionFactory;

    @Before
    public final void init(){
        try {
            Reader reader = getConfigFileAsReader();
            sqlSessionFactory = new SqlSessionFactoryBuilder().build(reader);
            reader.close();
            runSql(getSqlFileAsReader());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * 执行 Sql
     *
     * @param reader
     */
    protected void runSql(Reader reader) {
        if(reader == null){
            return;
        }
        SqlSession sqlSession = getSqlSession();
        try {
            Connection conn = sqlSession.getConnection();
            ScriptRunner runner = new ScriptRunner(conn);
            runner.setLogWriter(null);
            runner.runScript(reader);
            try {
                reader.close();
            } catch (IOException e) {}
        } finally {
            sqlSession.close();
        }
    }

    /**
     * 获取 mybatis 配置
     *
     * @return
     */
    protected Reader getConfigFileAsReader() throws IOException {
        URL url = BaseTest.class.getResource("mybatis-config.xml");
        return toReader(url);
    };

    /**
     * 获取初始化 sql
     *
     * @return
     */
    protected Reader getSqlFileAsReader() throws IOException {
        URL url = BaseTest.class.getResource("CreateDB.sql");
        return toReader(url);
    };

    /**
     * 转为 Reader
     *
     * @param url
     * @return
     * @throws IOException
     */
    protected Reader toReader(URL url) throws IOException {
        return Resources.getUrlAsReader(url.toString());
    }

    /**
     * 获取Session
     *
     * @return
     */
    protected SqlSession getSqlSession() {
        return sqlSessionFactory.openSession();
    }
}
